"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var system_general_dao_1 = require("../dao/system-general-dao");
var system_time_dao_1 = require("../dao/system-time-dao");
var system_dataset_dao_1 = require("../dao/system-dataset-dao");
var system_device_dao_1 = require("../dao/system-device-dao");
var system_section_dao_1 = require("../dao/system-section-dao");
var database_dao_1 = require("../dao/database-dao");
var system_advanced_dao_1 = require("../dao/system-advanced-dao");
var system_ui_dao_1 = require("../dao/system-ui-dao");
var debug_dao_1 = require("../dao/debug-dao");
var system_info_dao_1 = require("../dao/system-info-dao");
var crypto_certificate_dao_1 = require("../dao/crypto-certificate-dao");
var moment = require("moment-timezone");
var abstract_repository_1 = require("./abstract-repository");
var SystemGeneral_1 = require("../model/SystemGeneral");
var SystemTime_1 = require("../model/SystemTime");
var SystemAdvanced_1 = require("../model/SystemAdvanced");
var SystemUi_1 = require("../model/SystemUi");
var model_event_name_1 = require("../model-event-name");
var SystemRepository = (function (_super) {
    __extends(SystemRepository, _super);
    function SystemRepository(systemGeneralDao, systemTimeDao, systemDatasetDao, systemDeviceDao, systemSectionDao, databaseDao, debugDao, systemAdvancedDao, systemUiDao, systemInfoDao, cryptoCertificateDao) {
        var _this = _super.call(this, [
            SystemGeneral_1.SystemGeneral.getClassName(),
            SystemTime_1.SystemTime.getClassName(),
            SystemAdvanced_1.SystemAdvanced.getClassName(),
            SystemUi_1.SystemUi.getClassName()
        ]) || this;
        _this.systemGeneralDao = systemGeneralDao;
        _this.systemTimeDao = systemTimeDao;
        _this.systemDatasetDao = systemDatasetDao;
        _this.systemDeviceDao = systemDeviceDao;
        _this.systemSectionDao = systemSectionDao;
        _this.databaseDao = databaseDao;
        _this.debugDao = debugDao;
        _this.systemAdvancedDao = systemAdvancedDao;
        _this.systemUiDao = systemUiDao;
        _this.systemInfoDao = systemInfoDao;
        _this.cryptoCertificateDao = cryptoCertificateDao;
        return _this;
    }
    SystemRepository.getInstance = function () {
        if (!SystemRepository.instance) {
            SystemRepository.instance = new SystemRepository(new system_general_dao_1.SystemGeneralDao(), new system_time_dao_1.SystemTimeDao(), new system_dataset_dao_1.SystemDatasetDao(), new system_device_dao_1.SystemDeviceDao(), new system_section_dao_1.SystemSectionDao(), new database_dao_1.DatabaseDao(), new debug_dao_1.DebugDao(), new system_advanced_dao_1.SystemAdvancedDao(), new system_ui_dao_1.SystemUiDao(), new system_info_dao_1.SystemInfoDao(), new crypto_certificate_dao_1.CryptoCertificateDao());
        }
        return SystemRepository.instance;
    };
    SystemRepository.prototype.getGeneral = function () {
        return this.systemGeneralDao.get();
    };
    SystemRepository.prototype.getTime = function () {
        return this.systemTimeDao.get();
    };
    SystemRepository.prototype.getGmtOffset = function () {
        return Promise.all([
            this.getTime(),
            this.getGeneral()
        ]).spread(function (time, general) {
            return moment.tz(time.system_time.$date, general.timezone).format('Z');
        });
    };
    SystemRepository.prototype.getVersion = function () {
        return this.systemInfoDao.version();
    };
    SystemRepository.prototype.getDataset = function () {
        return this.systemDatasetDao.get();
    };
    SystemRepository.prototype.getDevices = function (deviceClass) {
        return this.systemDeviceDao.getDevices(deviceClass);
    };
    SystemRepository.prototype.listSystemSections = function () {
        return this.systemSectionDao.list();
    };
    SystemRepository.prototype.listTimezones = function () {
        return this.systemGeneralDao.listTimezones();
    };
    SystemRepository.prototype.listKeymaps = function () {
        return this.systemGeneralDao.listKeymaps();
    };
    SystemRepository.prototype.getConfigFileAddress = function () {
        return this.databaseDao.dump('freenas10.db');
    };
    SystemRepository.prototype.restoreFactorySettings = function () {
        return this.databaseDao.factoryRestore();
    };
    SystemRepository.prototype.restoreDatabase = function (file) {
        return this.databaseDao.restore(file);
    };
    SystemRepository.prototype.getAdvanced = function () {
        return this.systemAdvancedDao.get();
    };
    SystemRepository.prototype.saveGeneral = function (general) {
        return this.systemGeneralDao.save(general);
    };
    SystemRepository.prototype.saveAdvanced = function (advanced) {
        return this.systemAdvancedDao.save(advanced);
    };
    SystemRepository.prototype.getTimezones = function () {
        return this.systemGeneralDao.listTimezones();
    };
    SystemRepository.prototype.getKeymaps = function () {
        return this.systemGeneralDao.listKeymaps();
    };
    SystemRepository.prototype.getUi = function () {
        return this.systemUiDao.get();
    };
    SystemRepository.prototype.saveUi = function (ui) {
        return this.systemUiDao.save(ui);
    };
    SystemRepository.prototype.getDebugFileAddress = function () {
        return this.debugDao.collect('freenasdebug.tar.gz');
    };
    SystemRepository.prototype.getCertificateFileAddress = function (id, certTarFileName) {
        return this.cryptoCertificateDao.collect(id, certTarFileName);
    };
    SystemRepository.prototype.handleStateChange = function (name, state, overlay) {
        switch (name) {
            case SystemGeneral_1.SystemGeneral.getClassName():
                this.systemGeneral = this.dispatchModelEvents(this.systemGeneral, model_event_name_1.ModelEventName.SystemGeneral, state);
                break;
            case SystemTime_1.SystemTime.getClassName():
                this.systemTime = this.dispatchModelEvents(this.systemTime, model_event_name_1.ModelEventName.SystemTime, state);
                break;
            case SystemAdvanced_1.SystemAdvanced.getClassName():
                this.systemAdvanced = this.dispatchModelEvents(this.systemAdvanced, model_event_name_1.ModelEventName.SystemAdvanced, state);
                break;
            case SystemUi_1.SystemUi.getClassName():
                this.systemUi = this.dispatchModelEvents(this.systemUi, model_event_name_1.ModelEventName.SystemUi, state);
                break;
        }
    };
    SystemRepository.prototype.handleEvent = function (name, data) { };
    return SystemRepository;
}(abstract_repository_1.AbstractRepository));
exports.SystemRepository = SystemRepository;
