"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var abstract_repository_1 = require("./abstract-repository");
var model_1 = require("../model");
var model_event_name_1 = require("../model-event-name");
var vm_dao_1 = require("../dao/vm-dao");
var vm_config_dao_1 = require("../dao/vm-config-dao");
var vm_template_dao_1 = require("../dao/vm-template-dao");
var vm_readme_dao_1 = require("../dao/vm-readme-dao");
var vm_guest_info_dao_1 = require("../dao/vm-guest-info-dao");
var VmDeviceType_1 = require("../model/enumerations/VmDeviceType");
var VmDeviceVolumeType_1 = require("../model/enumerations/VmDeviceVolumeType");
var _ = require("lodash");
var uuid = require("uuid");
var vm_device_dao_1 = require("../dao/vm-device-dao");
var vm_volume_dao_1 = require("../dao/vm-volume-dao");
var vm_clone_dao_1 = require("../dao/vm-clone-dao");
var VmRepository = (function (_super) {
    __extends(VmRepository, _super);
    function VmRepository(vmDao, vmConfigDao, vmTemplateDao, vmReadmeDao, vmGuestInfoDao, vmDeviceDao, vmVolumeDao, vmCloneDao) {
        var _this = _super.call(this, [
            model_1.Model.Vm,
            model_1.Model.VmDatastore,
            model_1.Model.VmConfig,
            model_1.Model.VmTemplate,
            model_1.Model.VmGuestInfo
        ]) || this;
        _this.vmDao = vmDao;
        _this.vmConfigDao = vmConfigDao;
        _this.vmTemplateDao = vmTemplateDao;
        _this.vmReadmeDao = vmReadmeDao;
        _this.vmGuestInfoDao = vmGuestInfoDao;
        _this.vmDeviceDao = vmDeviceDao;
        _this.vmVolumeDao = vmVolumeDao;
        _this.vmCloneDao = vmCloneDao;
        _this.DEVICE_TYPE = _.omit(VmDeviceType_1.VmDeviceType, '_montage_metadata');
        _this.DATASTORE_TYPE = {
            NFS: 'nfs'
        };
        _this.BOOTABLE_DEVICE_TYPES = [
            'VOLUME',
            'CDROM',
            'DISK'
        ];
        _this.DEFAULT_CLONE_DEVICE_ID = 'TEMPLATE_DEVICE_ID';
        _this.DEFAULT_VM_CONFIG = {
            ncpus: '',
            bootloader: 'GRUB'
        };
        _this.DEFAULT_DEVICE_PROPERTIES = {
            CDROM: {},
            DISK: {
                mode: 'AHCI'
            },
            GRAPHICS: {
                resolution: '1024x768'
            },
            NIC: {
                mode: 'NAT',
                device: 'VIRTIO'
            },
            USB: {
                device: 'tablet'
            }
        };
        return _this;
    }
    VmRepository.getInstance = function () {
        if (!VmRepository.instance) {
            VmRepository.instance = new VmRepository(new vm_dao_1.VmDao(), new vm_config_dao_1.VmConfigDao(), new vm_template_dao_1.VmTemplateDao(), new vm_readme_dao_1.VmReadmeDao(), new vm_guest_info_dao_1.VmGuestInfoDao(), new vm_device_dao_1.VmDeviceDao(), new vm_volume_dao_1.VmVolumeDao(), new vm_clone_dao_1.VmCloneDao());
        }
        return VmRepository.instance;
    };
    VmRepository.prototype.listVms = function () {
        var _this = this;
        var promise = this.vms ? Promise.resolve(this.vms.valueSeq().toJS()) : this.vmDao.list();
        return promise.then(function (vms) {
            for (var _i = 0, vms_1 = vms; _i < vms_1.length; _i++) {
                var vm = vms_1[_i];
                vm._hasGraphicDevice = _.get(vm, 'devices').some(function (x) {
                    return x.type === _this.DEVICE_TYPE.GRAPHICS;
                });
            }
            return vms;
        });
    };
    VmRepository.prototype.getVmSettings = function () {
        var _this = this;
        var promise = this.vmConfig ? Promise.resolve(this.vmConfig.toJS()) : this.vmConfigDao.get();
        return promise.then(function (vmConfig) {
            _this.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName.VmConfig.contentChange, function (state) {
                _.assign(vmConfig, state.toJS());
            });
            return {
                config: vmConfig
            };
        });
    };
    VmRepository.prototype.getVmReadme = function (vm) {
        return this.vmReadmeDao.getNewInstance().then(function (vmReadme) {
            vmReadme.text = vm.config && vm.config.readme;
            return vmReadme;
        });
    };
    VmRepository.prototype.getNewVmClone = function () {
        return this.vmCloneDao.getNewInstance();
    };
    VmRepository.prototype.getGuestInfo = function (vm) {
        return this.vmGuestInfoDao.getGuestInfo(vm);
    };
    VmRepository.prototype.getNewVm = function () {
        return this.vmDao.getNewInstance();
    };
    VmRepository.prototype.saveVm = function (vm) {
        var vmPlain = _.toPlainObject(vm);
        vmPlain.devices = _.map(vmPlain.devices, function (device) { return _.omitBy(_.omit(_.toPlainObject(device), 'id'), _.isNull); });
        return this.vmDao.save(vmPlain);
    };
    VmRepository.prototype.cloneVmToName = function (vmId, name) {
        return this.vmCloneDao.clone(vmId, name);
    };
    VmRepository.prototype.getNewVmDeviceForType = function (type) {
        return type !== VmDeviceType_1.VmDeviceType.VOLUME &&
            this.vmDeviceDao.getNewInstance().then(function (device) { return VmRepository.setDeviceDefaults(device, type); });
    };
    VmRepository.prototype.cloneVmDevice = function (device) {
        return this.vmDeviceDao.getNewInstance().then(function (clone) {
            return _.assignWith(clone, device, { _isNew: false }, function (cloneValue, deviceValue, key) { return key === 'id' ? cloneValue : deviceValue; });
        });
    };
    VmRepository.prototype.getNewVmVolume = function () {
        return this.vmVolumeDao.getNewInstance().then(function (vmVolume) { return _.assign(vmVolume, {
            id: uuid.v4(),
            type: VmDeviceType_1.VmDeviceType.VOLUME
        }); });
    };
    VmRepository.prototype.flushTemplateCache = function () {
        return this.vmDao.flushTemplateCache();
    };
    VmRepository.initializeNewVmDevice = function (device) {
        if (device.type === VmDeviceType_1.VmDeviceType.VOLUME) {
            device.properties.type = VmDeviceVolumeType_1.VmDeviceVolumeType.VT9P;
        }
    };
    VmRepository.prototype.startVm = function (vm) {
        return this.vmDao.start(vm);
    };
    VmRepository.prototype.stopVm = function (vm, force) {
        if (force === void 0) { force = false; }
        return this.vmDao.stop(vm, force);
    };
    VmRepository.prototype.rebootVm = function (vm) {
        return this.vmDao.reboot(vm);
    };
    VmRepository.prototype.getSerialToken = function (vm) {
        return this.vmDao.requestSerialConsole(vm);
    };
    VmRepository.prototype.getWebVncConsoleUrl = function (vm) {
        return this.vmDao.requestWebvncConsole(vm);
    };
    VmRepository.setDeviceDefaults = function (device, type) {
        return _.assign(device, {
            _tmpId: type,
            type: type,
            properties: _.assign(device.properties, VmRepository.getDefaultPropertiesForDeviceType(type))
        });
    };
    VmRepository.getDefaultPropertiesForDeviceType = function (type) {
        var properties = {};
        switch (type) {
            case VmDeviceType_1.VmDeviceType.CDROM:
                break;
            case VmDeviceType_1.VmDeviceType.DISK:
                properties.mode = 'AHCI';
                properties.target_type = 'FILE';
                break;
            case VmDeviceType_1.VmDeviceType.GRAPHICS:
                properties.resolution = '1024x768';
                break;
            case VmDeviceType_1.VmDeviceType.NIC:
                properties.device = 'VIRTIO';
                properties.mode = 'NAT';
                break;
            case VmDeviceType_1.VmDeviceType.USB:
                properties.device = 'tablet';
                break;
        }
        return properties;
    };
    VmRepository.prototype.listTemplates = function () {
        return this.vmTemplates ? Promise.resolve(this.vmTemplates.valueSeq().toJS()) : this.vmTemplateDao.list();
    };
    VmRepository.prototype.getHardwareCapabilities = function () {
        return this.vmDao.getHardwareCapabilities();
    };
    VmRepository.prototype.saveVmSettings = function (vmSettings) {
        return this.vmConfigDao.save(vmSettings.config);
    };
    VmRepository.prototype.handleStateChange = function (name, state) {
        switch (name) {
            case model_1.Model.Vm:
                this.vms = this.dispatchModelEvents(this.vms, model_event_name_1.ModelEventName.Vm, state);
                break;
            case model_1.Model.VmDatastore:
                this.vmDatastores = this.dispatchModelEvents(this.vmDatastores, model_event_name_1.ModelEventName.VmDatastore, state);
                break;
            case model_1.Model.VmConfig:
                this.vmConfig = this.dispatchSingleObjectChange(this.vmConfig, model_event_name_1.ModelEventName.VmConfig, state);
                break;
            case model_1.Model.VmTemplate:
                this.vmTemplates = this.dispatchModelEvents(this.vmTemplates, model_event_name_1.ModelEventName.VmTemplate, state);
                break;
        }
    };
    VmRepository.prototype.handleEvent = function (name, data) {
    };
    return VmRepository;
}(abstract_repository_1.AbstractRepository));
exports.VmRepository = VmRepository;
