"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var model_event_name_1 = require("../model-event-name");
var account_repository_1 = require("../repository/account-repository");
var abstract_route_1 = require("./abstract-route");
var kerberos_repository_1 = require("../repository/kerberos-repository");
var model_1 = require("../model");
var AccountsRoute = (function (_super) {
    __extends(AccountsRoute, _super);
    function AccountsRoute(accountRepository, kerberosRepository) {
        var _this = _super.call(this) || this;
        _this.accountRepository = accountRepository;
        _this.kerberosRepository = kerberosRepository;
        return _this;
    }
    AccountsRoute.getInstance = function () {
        if (!AccountsRoute.instance) {
            AccountsRoute.instance = new AccountsRoute(account_repository_1.AccountRepository.getInstance(), kerberos_repository_1.KerberosRepository.getInstance());
        }
        return AccountsRoute.instance;
    };
    AccountsRoute.prototype.loadSection = function () {
        this.enterSection('accounts');
    };
    AccountsRoute.prototype.listUsers = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.User, columnIndex = 1, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/user'
        };
        return Promise.all([
            this.accountRepository.streamUsers(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (users, uiDescriptor) {
            var filter = { builtin: false }, sort = 'username';
            var filteredUsers = _.sortBy(_.filter(users, filter), sort);
            filteredUsers._objectType = objectType;
            filteredUsers._stream = users._stream;
            context.object = filteredUsers;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[objectType].listChange, function (state) {
                self.dataObjectChangeService.handleDataChange(filteredUsers, state, { filter: filter, sort: sort });
            });
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.getUser = function (userId) {
        var _this = this;
        var objectType = model_1.Model.User, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(userId)
        };
        return Promise.all([
            this.accountRepository.listUsers(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (users, uiDescriptor) {
            context.object = _.find(users, { id: userId });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.createUser = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.User, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.accountRepository.getNewUser(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (user, uiDescriptor) {
            context.object = user;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.listGroups = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.Group, columnIndex = 1, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/group'
        };
        return Promise.all([
            this.accountRepository.streamGroups(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (groups, uiDescriptor) {
            var filter = { builtin: false }, sort = 'name';
            var filteredGroups = _.sortBy(_.filter(groups, filter), sort);
            filteredGroups._objectType = objectType;
            filteredGroups._stream = groups._stream;
            context.object = filteredGroups;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[objectType].listChange, function (state) {
                self.dataObjectChangeService.handleDataChange(filteredGroups, state, { filter: filter, sort: sort });
            });
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.getGroup = function (groupId) {
        var _this = this;
        var self = this, objectType = model_1.Model.Group, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(groupId)
        };
        return Promise.all([
            this.accountRepository.listGroups(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (groups, uiDescriptor) {
            context.object = _.find(groups, { id: groupId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.createGroup = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.Group, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.accountRepository.getNewGroup(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (group, uiDescriptor) {
            context.object = group;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.listAccountSystems = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.AccountSystem, columnIndex = 1, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/account-system'
        };
        return Promise.all([
            this.accountRepository.listSystemUsersAndGroups(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (accountSystems, uiDescriptor) {
            var filter = { builtin: true }, sort = ['username', 'name'];
            context.object = _.sortBy(_.filter(accountSystems, filter), sort);
            context.object._objectType = context.objectType = model_1.Model.AccountSystem;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = [
                self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName.User.listChange, function (state) {
                    self.dataObjectChangeService.handleDataChange(accountSystems, state, model_1.Model.User, { filter: filter, sort: sort });
                }),
                self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName.Group.listChange, function (state) {
                    self.dataObjectChangeService.handleDataChange(accountSystems, state, model_1.Model.Group, { filter: filter, sort: sort });
                })
            ];
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.getDirectoryServices = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.DirectoryServices, columnIndex = 1, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/directory-services'
        };
        return Promise.all([
            this.accountRepository.getNewDirectoryServices(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (directoryServices, uiDescriptor) {
            context.object = directoryServices;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.getDirectory = function (directoryId) {
        var _this = this;
        var self = this, objectType = model_1.Model.Directory, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(directoryId)
        };
        return Promise.all([
            this.accountRepository.listDirectories(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (directories, uiDescriptor) {
            context.userInterfaceDescriptor = uiDescriptor;
            var directory = _.find(directories, { id: directoryId }), promise;
            if (directory) {
                promise = Promise.resolve(directory);
            }
            else {
                promise = self.accountRepository.getNewDirectoryForType(directoryId);
            }
            return promise;
        }).then(function (directory) {
            context.object = directory;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.listKerberosRealms = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.KerberosRealm, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/kerberos-realm'
        };
        return Promise.all([
            this.kerberosRepository.listKerberosRealms(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (kerberosRealms, uiDescriptor) {
            context.object = kerberosRealms;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[objectType].listChange, function (state) {
                self.dataObjectChangeService.handleDataChange(kerberosRealms, state);
            });
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.getKerberosRealm = function (kerberosRealmId) {
        var _this = this;
        var self = this, objectType = model_1.Model.KerberosRealm, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(kerberosRealmId)
        };
        return Promise.all([
            this.kerberosRepository.listKerberosRealms(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (kerberosRealms, uiDescriptor) {
            context.object = _.find(kerberosRealms, { id: kerberosRealmId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.createKerberosRealm = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.KerberosRealm, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.kerberosRepository.getNewKerberosRealm(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (kerberosRealm, uiDescriptor) {
            context.object = kerberosRealm;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.listKerberosKeytabs = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.KerberosKeytab, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/kerberos-keytab'
        };
        return Promise.all([
            this.kerberosRepository.listKerberosKeytabs(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (kerberosTabs, uiDescriptor) {
            context.object = kerberosTabs;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[objectType].listChange, function (state) {
                self.dataObjectChangeService.handleDataChange(kerberosTabs, state);
            });
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.getKerberosKeytab = function (kerberosKeytabId) {
        var _this = this;
        var self = this, objectType = model_1.Model.KerberosKeytab, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(kerberosKeytabId)
        };
        return Promise.all([
            this.kerberosRepository.listKerberosKeytabs(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (kerberosKeytabs, uiDescriptor) {
            context.object = _.find(kerberosKeytabs, { id: kerberosKeytabId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    AccountsRoute.prototype.createKerberosKeytab = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.KerberosKeytab, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.kerberosRepository.getNewKerberosKeytab(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (kerberosKeytab, uiDescriptor) {
            context.object = kerberosKeytab;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    return AccountsRoute;
}(abstract_route_1.AbstractRoute));
__decorate([
    abstract_route_1.Route('/accounts'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "loadSection", null);
__decorate([
    abstract_route_1.Route('/accounts/user'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "listUsers", null);
__decorate([
    abstract_route_1.Route('/accounts/user/_/{userId}'),
    abstract_route_1.Route('/accounts/account-system/user/_/{userId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "getUser", null);
__decorate([
    abstract_route_1.Route('/accounts/user/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "createUser", null);
__decorate([
    abstract_route_1.Route('/accounts/group'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "listGroups", null);
__decorate([
    abstract_route_1.Route('/accounts/group/_/{groupId}'),
    abstract_route_1.Route('/accounts/account-system/group/_/{groupId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "getGroup", null);
__decorate([
    abstract_route_1.Route('/accounts/group/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "createGroup", null);
__decorate([
    abstract_route_1.Route('/accounts/account-system'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "listAccountSystems", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "getDirectoryServices", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services/directory/_/{directoryId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "getDirectory", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services/kerberos-realm'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "listKerberosRealms", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services/kerberos-realm/_/{kerberosRealmId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "getKerberosRealm", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services/kerberos-realm/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "createKerberosRealm", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services/kerberos-keytab'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "listKerberosKeytabs", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services/kerberos-keytab/_/{kerberosKeytabId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "getKerberosKeytab", null);
__decorate([
    abstract_route_1.Route('/accounts/directory-services/kerberos-keytab/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], AccountsRoute.prototype, "createKerberosKeytab", null);
exports.AccountsRoute = AccountsRoute;
