"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var volume_repository_1 = require("../repository/volume-repository");
var model_event_name_1 = require("../model-event-name");
var vmware_repository_1 = require("../repository/vmware-repository");
var share_repository_1 = require("../repository/share-repository");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var _ = require("lodash");
var DatasetRoute = (function (_super) {
    __extends(DatasetRoute, _super);
    function DatasetRoute(volumeRepository, vmwareRepository, shareRepository) {
        var _this = _super.call(this) || this;
        _this.volumeRepository = volumeRepository;
        _this.vmwareRepository = vmwareRepository;
        _this.shareRepository = shareRepository;
        _this.objectType = model_1.Model.VolumeDataset;
        return _this;
    }
    DatasetRoute.getInstance = function () {
        if (!DatasetRoute.instance) {
            DatasetRoute.instance = new DatasetRoute(volume_repository_1.VolumeRepository.getInstance(), vmware_repository_1.VmwareRepository.getInstance(), share_repository_1.ShareRepository.getInstance());
        }
        return DatasetRoute.instance;
    };
    DatasetRoute.prototype.list = function (volumeId, stack) {
        var columnIndex = 2;
        return this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/volume-dataset', model_1.Model.VolumeDataset, this.volumeRepository.listDatasets(), {
            filter: { volume: volumeId },
            sort: 'name'
        });
    };
    DatasetRoute.prototype.create = function (volumeId, stack) {
        var self = this, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.volumeRepository.getNewVolumeDataset(),
            this.modelDescriptorService.getUiDescriptorForType(self.objectType)
        ]).spread(function (volumes, dataset, uiDescriptor) {
            dataset._volume = _.find(volumes, { id: volumeId });
            context.object = dataset;
            context.userInterfaceDescriptor = uiDescriptor;
            while (stack.length > columnIndex) {
                var context_1 = stack.pop();
                if (context_1 && context_1.changeListener) {
                    self.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName[context_1.objectType].listChange, context_1.changeListener);
                }
            }
            stack.push(context);
            return stack;
        });
    };
    DatasetRoute.prototype.get = function (volumeId, datasetId, stack) {
        var self = this, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(datasetId)
        };
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.volumeRepository.listDatasets(),
            this.shareRepository.listShares(),
            this.modelDescriptorService.getUiDescriptorForType(self.objectType)
        ]).spread(function (volumes, datasets, shares, uiDescriptor) {
            var dataset = _.find(datasets, { id: datasetId });
            dataset._volume = _.find(volumes, { id: volumeId });
            dataset._share = _.find(shares, function (share) {
                return (share.target_path === dataset.name && share.target_type === 'DATASET');
            });
            context.object = dataset;
            context.userInterfaceDescriptor = uiDescriptor;
            while (stack.length > columnIndex) {
                var context_2 = stack.pop();
                if (context_2 && context_2.changeListener) {
                    self.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName[context_2.objectType].listChange, context_2.changeListener);
                }
            }
            stack.push(context);
            return stack;
        });
    };
    DatasetRoute.prototype.listVmware = function (datasetId, stack) {
        var columnIndex = 4;
        return this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/vmware-dataset', model_1.Model.VmwareDataset, this.vmwareRepository.listDatasets(), {
            filter: { dataset: datasetId },
            sort: 'name'
        });
    };
    DatasetRoute.prototype.createVmware = function (datasetId, stack) {
        var self = this, columnIndex = 5, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.vmwareRepository.getNewVmwareDataset(),
            this.modelDescriptorService.getUiDescriptorForType('VmwareDataset')
        ]).spread(function (vmwareDataset, uiDescriptor) {
            vmwareDataset.dataset = datasetId;
            context.object = vmwareDataset;
            context.userInterfaceDescriptor = uiDescriptor;
            while (stack.length > columnIndex) {
                var context_3 = stack.pop();
                if (context_3 && context_3.changeListener) {
                    self.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName[context_3.objectType].listChange, context_3.changeListener);
                }
            }
            stack.push(context);
            return stack;
        });
    };
    DatasetRoute.prototype.getVmware = function (vmwareDatasetId, stack) {
        var self = this, columnIndex = 5, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(vmwareDatasetId)
        };
        return Promise.all([
            this.vmwareRepository.listDatasets(),
            this.modelDescriptorService.getUiDescriptorForType('VmwareDataset')
        ]).spread(function (vmwareDatasets, uiDescriptor) {
            context.object = _.find(vmwareDatasets, { id: vmwareDatasetId });
            context.userInterfaceDescriptor = uiDescriptor;
            while (stack.length > columnIndex) {
                var context_4 = stack.pop();
                if (context_4 && context_4.changeListener) {
                    self.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName[context_4.objectType].listChange, context_4.changeListener);
                }
            }
            stack.push(context);
            return stack;
        });
    };
    DatasetRoute.prototype.getShare = function (volumeId, datasetId, stack) {
        var self = this, objectType = model_1.Model.Share;
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.shareRepository.listShares(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (volumes, shares, uiDescriptor) {
            var columnIndex = 4;
            while (stack.length > columnIndex) {
                var context = stack.pop();
                if (context && context.changeListener) {
                    self.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName[context.objectType].listChange, context.changeListener);
                }
            }
            var share = _.find(shares, function (share) {
                return (share.target_path === datasetId && share.target_type === 'DATASET');
            });
            share._volume = _.find(volumes, { id: volumeId });
            stack.push({
                object: share,
                userInterfaceDescriptor: uiDescriptor,
                columnIndex: columnIndex,
                objectType: objectType,
                parentContext: stack[columnIndex - 1],
                path: stack[columnIndex - 1].path + '/share'
            });
            return stack;
        });
    };
    return DatasetRoute;
}(abstract_route_1.AbstractRoute));
exports.DatasetRoute = DatasetRoute;
