"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var model_event_name_1 = require("../model-event-name");
var docker_host_repository_1 = require("../repository/docker-host-repository");
var docker_image_readme_repository_1 = require("../repository/docker-image-readme-repository");
var docker_image_repository_1 = require("../repository/docker-image-repository");
var docker_collection_repository_1 = require("../repository/docker-collection-repository");
var docker_container_repository_1 = require("../repository/docker-container-repository");
var docker_network_repository_1 = require("../repository/docker-network-repository");
var docker_container_logs_repository_1 = require("../repository/docker-container-logs-repository");
var DockerContainer_1 = require("../model/DockerContainer");
var DockerNetwork_1 = require("../model/DockerNetwork");
var vm_repository_1 = require("../repository/vm-repository");
var DockerRoute = (function (_super) {
    __extends(DockerRoute, _super);
    function DockerRoute(dockerHostRepository, dockerImageRepository, dockerCollectionRepository, dockerContainerRepository, dockerNetworkRepository, dockerContainerLogsRepository, dockerImageReadmeRepository, vmRepository) {
        var _this = _super.call(this) || this;
        _this.dockerHostRepository = dockerHostRepository;
        _this.dockerImageRepository = dockerImageRepository;
        _this.dockerCollectionRepository = dockerCollectionRepository;
        _this.dockerContainerRepository = dockerContainerRepository;
        _this.dockerNetworkRepository = dockerNetworkRepository;
        _this.dockerContainerLogsRepository = dockerContainerLogsRepository;
        _this.dockerImageReadmeRepository = dockerImageReadmeRepository;
        _this.vmRepository = vmRepository;
        return _this;
    }
    DockerRoute.getInstance = function () {
        if (!DockerRoute.instance) {
            DockerRoute.instance = new DockerRoute(docker_host_repository_1.DockerHostRepository.getInstance(), docker_image_repository_1.DockerImageRepository.getInstance(), docker_collection_repository_1.DockerCollectionRepository.getInstance(), docker_container_repository_1.DockerContainerRepository.getInstance(), docker_network_repository_1.DockerNetworkRepository.getInstance(), docker_container_logs_repository_1.DockerContainerLogsRepository.getInstance(), docker_image_readme_repository_1.DockerImageReadmeRepository.getInstance(), vm_repository_1.VmRepository.getInstance());
        }
        return DockerRoute.instance;
    };
    DockerRoute.prototype.listHosts = function (stack) {
        var _this = this;
        var self = this, objectType = model_1.Model.DockerHost, columnIndex = 1, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/docker-host'
        };
        return Promise.all([
            this.dockerHostRepository.listDockerHosts(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (hosts, uiDescriptor) {
            context.object = hosts;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[objectType].listChange, function (state) {
                self.dataObjectChangeService.handleDataChange(hosts, state);
            });
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.getHost = function (hostId, stack) {
        var _this = this;
        var objectType = model_1.Model.DockerHost, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + hostId
        };
        return Promise.all([
            this.dockerHostRepository.listDockerHosts(),
            this.vmRepository.listVms(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (hosts, vms, uiDescriptor) {
            context.object = _.find(hosts, { id: hostId });
            context.object._vm = _.find(vms, { id: hostId });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.listDockerNetworks = function (hostId, stack) {
        var columnIndex = 3;
        return this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/docker-network', DockerNetwork_1.DockerNetwork.getClassName(), this.dockerNetworkRepository.list(), {
            filter: { host: hostId }
        });
    };
    DockerRoute.prototype.getDockerNetwork = function (dockerNetworkId, stack) {
        var columnIndex = 4;
        return Promise.all([
            this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/create', DockerNetwork_1.DockerNetwork.getClassName(), this.dockerNetworkRepository.list(), {
                id: dockerNetworkId
            }),
            this.dockerContainerRepository.list()
        ]).spread(function (stack, containers) {
            var network = _.last(stack).object;
            network._containers = _.filter(containers, { host: network.host });
            return stack;
        });
    };
    DockerRoute.prototype.createDockerNetwork = function (hostId, stack) {
        var columnIndex = 4;
        return Promise.all([
            this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/create', DockerNetwork_1.DockerNetwork.getClassName(), this.dockerNetworkRepository.getNewInstance()),
            this.dockerContainerRepository.list()
        ]).spread(function (stack, containers) {
            var network = _.last(stack).object;
            network.host = hostId;
            network._containers = _.filter(containers, { host: hostId });
            return stack;
        });
    };
    DockerRoute.prototype.createHost = function (stack) {
        var _this = this;
        var objectType = model_1.Model.DockerHost, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.dockerHostRepository.getNewDockerHost(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (dockerHost, uiDescriptor) {
            context.object = dockerHost;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.listImages = function (stack) {
        var _this = this;
        var self = this, objectType = model_1.Model.DockerImage, columnIndex = 1, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/docker-image'
        };
        return Promise.all([
            this.dockerImageRepository.listDockerImages(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (images, uiDescriptor) {
            context.object = images;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[objectType].listChange, function (state) {
                self.dataObjectChangeService.handleDataChange(images, state);
            });
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.getImage = function (imageId, stack) {
        var _this = this;
        var objectType = model_1.Model.DockerImage, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + imageId
        };
        return Promise.all([
            this.dockerImageRepository.listDockerImages(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (images, uiDescriptor) {
            context.object = _.find(images, { id: imageId });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.pullImage = function (collectionId, stack) {
        var _this = this;
        var objectType = model_1.Model.DockerImagePull, columnIndex = 2, parentContext = stack[columnIndex], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path
        };
        return Promise.all([
            this.dockerImageRepository.getNewDockerImage(),
            this.dockerCollectionRepository.listDockerCollections(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (image, collections, uiDescriptor) {
            image.dockerCollection = _.find(collections, { id: collectionId });
            image._isNewObject = true;
            context.userInterfaceDescriptor = uiDescriptor;
            context.object = image;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.listCollections = function (stack) {
        var _this = this;
        var self = this, objectType = model_1.Model.DockerCollection, columnIndex = 1, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/docker-collection'
        };
        return Promise.all([
            this.dockerCollectionRepository.listDockerCollections(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (images, uiDescriptor) {
            context.object = images;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = self.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[objectType].listChange, function (state) {
                self.dataObjectChangeService.handleDataChange(images, state);
            });
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.getCollection = function (collectionId, stack) {
        var _this = this;
        var objectType = model_1.Model.DockerCollection, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + collectionId
        };
        return Promise.all([
            this.dockerCollectionRepository.listDockerCollections(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (collections, uiDescriptor) {
            context.object = _.find(collections, { id: collectionId });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.listContainers = function (stack) {
        var columnIndex = 1;
        return this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/docker-container', DockerContainer_1.DockerContainer.getClassName(), this.dockerContainerRepository.list());
    };
    DockerRoute.prototype.getContainer = function (containerId, stack) {
        var columnIndex = 2;
        return Promise.all(_.flatten([
            this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/_/' + containerId, DockerContainer_1.DockerContainer.getClassName(), this.dockerContainerRepository.list(), { id: containerId }),
            this.getContainerDependencies()
        ])).spread(function (stack, hosts, networks, networkModes) {
            var container = _.last(stack).object;
            container._hosts = hosts;
            container._networks = networks;
            container._networkModes = networkModes;
            return stack;
        });
    };
    DockerRoute.prototype.getSettings = function () {
        // todo
    };
    DockerRoute.prototype.listCollectionsForCreate = function (stack) {
        var _this = this;
        var objectType = model_1.Model.DockerCollection, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create',
            isCreatePrevented: true
        };
        return Promise.all([
            this.dockerCollectionRepository.listDockerCollections(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (collections, uiDescriptor) {
            context.object = collections;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.createCollection = function (stack) {
        var _this = this;
        var objectType = model_1.Model.DockerCollection, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.dockerCollectionRepository.getNewDockerCollection(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (collection, uiDescriptor) {
            context.userInterfaceDescriptor = uiDescriptor;
            context.object = collection;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.createContainer = function (collectionId, stack) {
        var _this = this;
        var objectType = model_1.Model.DockerContainer, columnIndex = 2, parentContext = stack[columnIndex], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/' + collectionId
        };
        return Promise.all(_.flatten([
            this.dockerContainerRepository.getNewDockerContainer(),
            this.getContainerDependencies(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ])).spread(function (container, hosts, networks, networkModes, collections, uiDescriptor) {
            container._collection = _.find(collections, { id: collectionId });
            container._imagesPromise = _this.dockerCollectionRepository.getDockerImagesWithCollection(container._collection);
            container._hosts = hosts;
            container._networks = networks;
            container._networkModes = networkModes;
            context.object = container;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.getReadmeForImage = function (imageName, stack) {
        var _this = this;
        var objectType = model_1.Model.DockerImageReadme, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/readme'
        };
        return Promise.all([
            this.dockerImageReadmeRepository.getDockerImageReadme(),
            this.dockerImageRepository.getReadmeForDockerImage(imageName),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (dockerImageReadme, readme, uiDescriptor) {
            dockerImageReadme.text = readme;
            dockerImageReadme.imageName = imageName;
            context.object = dockerImageReadme;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.getReadmeForContainer = function (containerId, stack) {
        var _this = this;
        var objectType = model_1.Model.DockerImageReadme, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/readme'
        };
        return this.dockerContainerRepository.list().then(function (containers) {
            var container = _.find(containers, { id: containerId });
            return Promise.all([
                _this.dockerImageReadmeRepository.getDockerImageReadme(),
                container.image,
                _this.dockerImageRepository.getReadmeForDockerImage(container.image),
                _this.modelDescriptorService.getUiDescriptorForType(objectType)
            ]);
        }).spread(function (dockerImageReadme, imageName, readme, uiDescriptor) {
            dockerImageReadme.imageName = imageName;
            dockerImageReadme.text = readme || 'No Readme available for this container';
            context.object = dockerImageReadme;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.getContainerLogs = function (stack) {
        var _this = this;
        var objectType = model_1.Model.DockerContainerLogs, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/docker-container-logs'
        };
        return Promise.all([
            this.dockerContainerLogsRepository.getNewDockerContainerLogs(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (dockerContainerLogs, uiDescriptor) {
            context.object = dockerContainerLogs;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(stack, context);
        });
    };
    DockerRoute.prototype.getContainerDependencies = function () {
        return [
            this.dockerHostRepository.listDockerHosts(),
            this.dockerNetworkRepository.list(),
            Promise.resolve(docker_network_repository_1.DockerNetworkRepository.PRIMARY_NETWORK_MODES),
            this.dockerCollectionRepository.listDockerCollections()
        ];
    };
    return DockerRoute;
}(abstract_route_1.AbstractRoute));
exports.DockerRoute = DockerRoute;
