"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var service_repository_1 = require("../repository/service-repository");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var model_event_name_1 = require("../model-event-name");
var ServicesRoute = (function (_super) {
    __extends(ServicesRoute, _super);
    function ServicesRoute(serviceRepository) {
        var _this = _super.call(this) || this;
        _this.serviceRepository = serviceRepository;
        return _this;
    }
    ServicesRoute.getInstance = function () {
        if (!ServicesRoute.instance) {
            ServicesRoute.instance = new ServicesRoute(service_repository_1.ServiceRepository.getInstance());
        }
        return ServicesRoute.instance;
    };
    ServicesRoute.prototype.getCategory = function (categoryId, stack) {
        var _this = this;
        var columnIndex = 1, parentContext = stack[0], context = {
            columnIndex: columnIndex,
            objectType: model_1.Model.ServicesCategory,
            parentContext: parentContext,
            path: parentContext.path + '/services-category/_/' + encodeURIComponent(categoryId)
        };
        return Promise.all([
            this.serviceRepository.listServicesCategories(),
            this.serviceRepository.listServices(),
            this.modelDescriptorService.getUiDescriptorForType(model_1.Model.ServicesCategory)
        ]).spread(function (categories, objects, uiDescriptor) {
            var category = _.find(categories, { id: categoryId }), options = {
                filter: function (service) {
                    return _.includes(category.types, 'service-' + service.name);
                },
                sort: 'name'
            }, filteredObjects = _.sortBy(_.filter(objects, options.filter), options.sort);
            context.objectType = filteredObjects._objectType = model_1.Model.ServicesCategory;
            context.object = filteredObjects;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = _this.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName[model_1.Model.Service].listChange, function (state) {
                _this.dataObjectChangeService.handleDataChange(filteredObjects, state, options);
            });
            return _this.updateStackWithContext(stack, context);
        });
    };
    ServicesRoute.prototype.getService = function (serviceId, stack) {
        var self = this, objectType = model_1.Model.Service, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/service/_/' + encodeURIComponent(serviceId)
        };
        return Promise.all([
            this.serviceRepository.listServices(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (services, uiDescriptor) {
            context.object = _.find(services, { id: serviceId });
            context.userInterfaceDescriptor = uiDescriptor;
            return Promise.resolve(context.object.config);
        }).then(function () {
            return self.updateStackWithContext(stack, context);
        });
    };
    ServicesRoute.prototype.listRsyncdModules = function (stack) {
        var self = this, objectType = model_1.Model.RsyncdModule, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/modules'
        };
        return this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/modules', model_1.Model.RsyncdModule, this.serviceRepository.listRsyncdModules());
    };
    ServicesRoute.prototype.createRsyncdModule = function (stack) {
        var self = this, objectType = model_1.Model.RsyncdModule, columnIndex = 4, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.serviceRepository.getNewRsyncdModule(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (rsyncdModule, uiDescriptor) {
            context.object = rsyncdModule;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    ServicesRoute.prototype.getRsyncdModule = function (rsyncdModuleId, stack) {
        var self = this, objectType = model_1.Model.RsyncdModule, columnIndex = 4, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/modules/rsyncd-module/_/' + encodeURIComponent(rsyncdModuleId)
        };
        return Promise.all([
            this.serviceRepository.listRsyncdModules(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (rsyncdModules, uiDescriptor) {
            context.object = _.find(rsyncdModules, { id: rsyncdModuleId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    return ServicesRoute;
}(abstract_route_1.AbstractRoute));
exports.ServicesRoute = ServicesRoute;
