"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var volume_repository_1 = require("../repository/volume-repository");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var SnapshotRoute = (function (_super) {
    __extends(SnapshotRoute, _super);
    function SnapshotRoute(volumeRepository) {
        var _this = _super.call(this) || this;
        _this.volumeRepository = volumeRepository;
        _this.objectType = model_1.Model.VolumeSnapshot;
        return _this;
    }
    SnapshotRoute.getInstance = function () {
        if (!SnapshotRoute.instance) {
            SnapshotRoute.instance = new SnapshotRoute(volume_repository_1.VolumeRepository.getInstance());
        }
        return SnapshotRoute.instance;
    };
    SnapshotRoute.prototype.list = function (volumeId, stack) {
        var columnIndex = 2;
        var streamSnapshots = this.volumeRepository.streamSnapshots({ volume: volumeId });
        return Promise.all([
            this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/volume-snapshot', model_1.Model.VolumeSnapshot, streamSnapshots, {
                filter: { volume: volumeId }
            }),
            streamSnapshots
        ]).spread(function (stack, snapshots) {
            _.last(stack).object._stream = snapshots._stream;
            return stack;
        });
    };
    SnapshotRoute.prototype.listForDataset = function (volumeId, datasetId, stack) {
        var columnIndex = 4;
        var streamSnapshots = this.volumeRepository.streamSnapshots({ volume: volumeId, dataset: datasetId });
        return Promise.all([
            this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/volume-snapshot', model_1.Model.VolumeSnapshot, streamSnapshots, {
                filter: { volume: volumeId, dataset: datasetId }
            }),
            streamSnapshots
        ]).spread(function (stack, snapshots) {
            _.last(stack).object._stream = snapshots._stream;
            return stack;
        });
    };
    SnapshotRoute.prototype.create = function (volumeId, stack) {
        var self = this, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.volumeRepository.getNewVolumeSnapshot(),
            this.modelDescriptorService.getUiDescriptorForType(self.objectType)
        ]).spread(function (volumes, snapshot, uiDescriptor) {
            snapshot._volume = _.find(volumes, { id: volumeId });
            context.object = snapshot;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SnapshotRoute.prototype.createForDataset = function (volumeId, datasetId, stack) {
        var self = this, columnIndex = 5, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.volumeRepository.listDatasets(),
            this.volumeRepository.getNewVolumeSnapshot(),
            this.modelDescriptorService.getUiDescriptorForType(self.objectType)
        ]).spread(function (volumes, datasets, snapshot, uiDescriptor) {
            snapshot._volume = _.find(volumes, { id: volumeId });
            snapshot._dataset = _.find(datasets, { id: datasetId });
            context.object = snapshot;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SnapshotRoute.prototype.get = function (volumeId, snapshotId, stack) {
        var columnIndex = 3;
        this.openSnapshotAtColumnIndex(volumeId, snapshotId, columnIndex, stack);
    };
    SnapshotRoute.prototype.getForDataset = function (volumeId, snapshotId, stack) {
        var columnIndex = 5;
        this.openSnapshotAtColumnIndex(volumeId, snapshotId, columnIndex, stack);
    };
    SnapshotRoute.prototype.openSnapshotAtColumnIndex = function (volumeId, snapshotId, columnIndex, stack) {
        var self = this, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(snapshotId)
        };
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.volumeRepository.listSnapshots(),
            this.modelDescriptorService.getUiDescriptorForType(self.objectType)
        ]).spread(function (volumes, snapshots, uiDescriptor) {
            var snapshot = _.find(snapshots, { id: snapshotId });
            snapshot._volume = _.find(volumes, { id: volumeId });
            context.object = snapshot;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    return SnapshotRoute;
}(abstract_route_1.AbstractRoute));
exports.SnapshotRoute = SnapshotRoute;
