"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var system_repository_1 = require("../repository/system-repository");
var CryptoCertificateType_1 = require("../model/enumerations/CryptoCertificateType");
var crypto_certificate_repository_1 = require("../repository/crypto-certificate-repository");
var alert_filter_repository_1 = require("../repository/alert-filter-repository");
var tunable_repository_1 = require("../repository/tunable-repository");
var ntp_server_repository_1 = require("../repository/ntp-server-repository");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var _ = require("lodash");
var alert_emitter_repository_1 = require("../repository/alert-emitter-repository");
var SystemRoute = (function (_super) {
    __extends(SystemRoute, _super);
    function SystemRoute(systemRepository, cryptoCertificateRepository, alertFilterRepository, tunableRepository, ntpServerRepository, alertEmitterRepository) {
        var _this = _super.call(this) || this;
        _this.systemRepository = systemRepository;
        _this.cryptoCertificateRepository = cryptoCertificateRepository;
        _this.alertFilterRepository = alertFilterRepository;
        _this.tunableRepository = tunableRepository;
        _this.ntpServerRepository = ntpServerRepository;
        _this.alertEmitterRepository = alertEmitterRepository;
        _this.objectType = model_1.Model.SystemSection;
        return _this;
    }
    SystemRoute.getInstance = function () {
        if (!SystemRoute.instance) {
            SystemRoute.instance = new SystemRoute(system_repository_1.SystemRepository.getInstance(), crypto_certificate_repository_1.CryptoCertificateRepository.getInstance(), alert_filter_repository_1.AlertFilterRepository.getInstance(), tunable_repository_1.TunableRepository.getInstance(), ntp_server_repository_1.NtpServerRepository.getInstance(), alert_emitter_repository_1.AlertEmitterRepository.getInstance());
        }
        return SystemRoute.instance;
    };
    SystemRoute.prototype.get = function (systemSectionId, stack) {
        var self = this, columnIndex = 1, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/system-section/_/' + systemSectionId
        };
        return Promise.all([
            this.systemRepository.listSystemSections(),
            this.modelDescriptorService.getUiDescriptorForType(this.objectType)
        ]).spread(function (systemSections, uiDescriptor) {
            context.object = _.find(systemSections, { id: systemSectionId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.getCertificate = function (certificateId, stack) {
        var self = this, objectType = model_1.Model.CryptoCertificate, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/crypto-certificate/_/' + encodeURIComponent(certificateId)
        };
        return Promise.all([
            this.cryptoCertificateRepository.listCryptoCertificates(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (certificates, uiDescriptor) {
            context.object = _.find(certificates, { id: certificateId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.selectNewCertificateType = function (stack) {
        var _this = this;
        var self = this, objectType = model_1.Model.CryptoCertificate, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            isCreatePrevented: true,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            Promise.all(_.map(_.keys(CryptoCertificateType_1.CryptoCertificateType), function (type) { return _this.cryptoCertificateRepository.getNewCryptoCertificate(type); })),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (cryptoCertificates, uiDescriptor) {
            cryptoCertificates._objectType = objectType;
            context.object = _.compact(cryptoCertificates);
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.createCertificate = function (certificateType, stack) {
        var self = this, objectType = model_1.Model.CryptoCertificate, columnIndex = 2, parentContext = stack[columnIndex], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/' + certificateType
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            var share = _.find(parentContext.object, { _tmpId: certificateType });
            context.userInterfaceDescriptor = uiDescriptor;
            context.object = share;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.getAlert = function (stack) {
        var self = this, objectType = model_1.Model.AlertFilter, columnIndex = 1, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/system-section/_/alert'
        };
        return Promise.all([
            this.alertFilterRepository.listAlertFilters(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (alertFilters, uiDescriptor) {
            context.object = {
                filters: alertFilters
            };
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.createAlertFilter = function (stack) {
        var self = this, objectType = model_1.Model.AlertFilter, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.alertFilterRepository.getNewAlertFilter(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (alertFilter, uiDescriptor) {
            context.object = alertFilter;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.getAlertSettings = function (stack) {
        var self = this, objectType = model_1.Model.AlertEmitter, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/settings'
        };
        return Promise.all([
            this.alertEmitterRepository.list(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (alertEmitters, uiDescriptor) {
            context.object = _.fromPairs(_.map(alertEmitters, function (alertEmitter) { return [alertEmitter.name, alertEmitter]; }));
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.createTunable = function (stack) {
        var self = this, objectType = model_1.Model.Tunable, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.tunableRepository.getNewTunable(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (tunable, uiDescriptor) {
            context.object = tunable;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.getTunable = function (tunableId, stack) {
        var self = this, objectType = model_1.Model.Tunable, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/tunable/_/' + encodeURIComponent(tunableId)
        };
        return Promise.all([
            this.tunableRepository.listTunables(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (tunables, uiDescriptor) {
            context.object = _.find(tunables, { id: tunableId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.createNtpServer = function (stack) {
        var self = this, objectType = model_1.Model.NtpServer, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.ntpServerRepository.getNewNtpServer(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (ntpServer, uiDescriptor) {
            context.object = ntpServer;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    SystemRoute.prototype.getNtpServer = function (ntpServerId, stack) {
        var self = this, objectType = model_1.Model.NtpServer, columnIndex = 2, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/ntp-server/_/' + encodeURIComponent(ntpServerId)
        };
        return Promise.all([
            this.ntpServerRepository.listNtpServers(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (ntpServers, uiDescriptor) {
            context.object = _.find(ntpServers, { id: ntpServerId });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    return SystemRoute;
}(abstract_route_1.AbstractRoute));
exports.SystemRoute = SystemRoute;
