"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var datastore_service_1 = require("../datastore-service");
var _ = require("lodash");
var immutable_1 = require("immutable");
var DiffProcessor = (function () {
    function DiffProcessor(datastoreService) {
        this.datastoreService = datastoreService || datastore_service_1.DatastoreService.getInstance();
    }
    DiffProcessor.prototype.process = function (object, type, id, propertyDescriptors) {
        if (_.isEmpty(object)) {
            return null;
        }
        var state = this.datastoreService.getState(), reference = state.has(type) && state.get(type).has(id) && state.get(type).get(id).toJS();
        return this.getDifferences(object, reference, propertyDescriptors);
    };
    DiffProcessor.prototype.getDifferences = function (object, reference, propertyDescriptors) {
        var differences;
        if (!reference) {
            differences = immutable_1.Map(object);
        }
        else {
            var usedReference = propertyDescriptors || reference;
            differences = immutable_1.Map();
            _.forEach(usedReference, function (value, key) {
                if (object.hasOwnProperty(key) && !_.isEqual(reference[key], object[key])) {
                    differences = differences.set(key, object[key]);
                }
            });
        }
        return differences.size > 0 ? differences.toJS() : null;
    };
    return DiffProcessor;
}());
var processor = new DiffProcessor();
exports.processor = processor;
