"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var EventDispatcherService = (function () {
    function EventDispatcherService() {
        this.listeners = new Map();
    }
    EventDispatcherService.getInstance = function () {
        return EventDispatcherService.instance || (EventDispatcherService.instance = new EventDispatcherService());
    };
    EventDispatcherService.prototype.addEventListener = function (eventName, handler) {
        if (!this.listeners.has(eventName)) {
            this.listeners.set(eventName, new Set());
        }
        this.listeners.get(eventName).add(handler);
        handler.eventName = eventName;
        return handler;
    };
    EventDispatcherService.prototype.removeEventListener = function (eventName, handler) {
        if (this.listeners.has(eventName)) {
            this.listeners.get(eventName).delete(handler);
        }
    };
    EventDispatcherService.prototype.dispatch = function (eventName, detail) {
        var promise;
        if (this.listeners.has(eventName)) {
            var handlers = this.listeners.get(eventName);
            promise = Promise.all(_.map(Array.from(handlers), function (handler) { return handler.call({}, detail); }))
                .then(function (results) { return _.some(results); });
        }
        else {
            promise = Promise.resolve();
        }
        return promise;
    };
    return EventDispatcherService;
}());
exports.EventDispatcherService = EventDispatcherService;
