"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var abstract_section_service_ng_1 = require("./abstract-section-service-ng");
var system_repository_1 = require("../../repository/system-repository");
var ntp_server_repository_1 = require("../../repository/ntp-server-repository");
var vm_repository_1 = require("../../repository/vm-repository");
var network_repository_1 = require("../../repository/network-repository");
var crypto_certificate_repository_1 = require("../../repository/crypto-certificate-repository");
var tunable_repository_1 = require("../../repository/tunable-repository");
var volume_repository_1 = require("../../repository/volume-repository");
var share_repository_1 = require("../../repository/share-repository");
var peer_repository_1 = require("../../repository/peer-repository");
var replication_repository_1 = require("../../repository/replication-repository");
var disk_repository_1 = require("../../repository/disk-repository");
var alert_filter_repository_1 = require("../../repository/alert-filter-repository");
var boot_pool_repository_1 = require("../../repository/boot-pool-repository");
var alert_emitter_push_bullet_repository_1 = require("../../repository/alert-emitter-push-bullet-repository");
var alert_emitter_repository_1 = require("../../repository/alert-emitter-repository");
var _ = require("lodash");
var docker_container_repository_1 = require("../../repository/docker-container-repository");
var SystemSectionService = (function (_super) {
    __extends(SystemSectionService, _super);
    function SystemSectionService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.SELF_SIGNED = crypto_certificate_repository_1.CryptoCertificateRepository.SELF_SIGNED;
        _this.CREATION = crypto_certificate_repository_1.CryptoCertificateRepository.CREATION;
        return _this;
    }
    SystemSectionService.prototype.init = function () {
        this.systemRepository = system_repository_1.SystemRepository.getInstance();
        this.ntpServerRepository = ntp_server_repository_1.NtpServerRepository.getInstance();
        this.vmRepository = vm_repository_1.VmRepository.getInstance();
        this.dockerContainerRepository = docker_container_repository_1.DockerContainerRepository.getInstance();
        this.networkRepository = network_repository_1.NetworkRepository.getInstance();
        this.cryptoCertificateRepository = crypto_certificate_repository_1.CryptoCertificateRepository.getInstance();
        this.tunableRepository = tunable_repository_1.TunableRepository.getInstance();
        this.volumeRepository = volume_repository_1.VolumeRepository.getInstance();
        this.shareRepository = share_repository_1.ShareRepository.getInstance();
        this.peerRepository = peer_repository_1.PeerRepository.getInstance();
        this.replicationRepository = replication_repository_1.ReplicationRepository.getInstance();
        this.diskRepository = disk_repository_1.DiskRepository.getInstance();
        this.bootPoolRepository = boot_pool_repository_1.BootPoolRepository.getInstance();
        this.alertFilterRepository = alert_filter_repository_1.AlertFilterRepository.getInstance();
        this.alertEmitterPushBulletRepository = alert_emitter_push_bullet_repository_1.AlertEmitterPushBulletRepository.getInstance();
        this.alertEmitterRepository = alert_emitter_repository_1.AlertEmitterRepository.getInstance();
    };
    SystemSectionService.prototype.loadEntries = function () {
        return this.systemRepository.listSystemSections();
    };
    SystemSectionService.prototype.getTimezoneOptions = function () {
        return this.systemRepository.getTimezones();
    };
    SystemSectionService.prototype.getKeymapOptions = function () {
        return this.systemRepository.getKeymaps();
    };
    SystemSectionService.prototype.getSystemGeneral = function () {
        return this.systemRepository.getGeneral();
    };
    SystemSectionService.prototype.saveSystemGeneral = function (systemGeneral) {
        return this.systemRepository.saveGeneral(systemGeneral);
    };
    SystemSectionService.prototype.getSystemTime = function () {
        return this.systemRepository.getTime();
    };
    SystemSectionService.prototype.getSystemVersion = function () {
        return this.systemRepository.getVersion();
    };
    SystemSectionService.prototype.getSystemDataset = function () {
        return this.systemRepository.getDataset();
    };
    SystemSectionService.prototype.getSystemAdvanced = function () {
        return this.systemRepository.getAdvanced();
    };
    SystemSectionService.prototype.saveSystemAdvanced = function (systemAdvanced) {
        return this.systemRepository.saveAdvanced(systemAdvanced);
    };
    SystemSectionService.prototype.listNtpServers = function () {
        return this.ntpServerRepository.listNtpServers();
    };
    SystemSectionService.prototype.saveNtpServer = function (ntpServer, force) {
        return this.ntpServerRepository.saveNtpServer(ntpServer, force);
    };
    SystemSectionService.prototype.listVms = function () {
        return this.vmRepository.listVms();
    };
    SystemSectionService.prototype.listContainers = function () {
        return this.dockerContainerRepository.list();
    };
    SystemSectionService.prototype.listNetworkInterfaces = function () {
        return this.networkRepository.listNetworkInterfaces();
    };
    SystemSectionService.prototype.listCertificates = function () {
        return this.cryptoCertificateRepository.listCryptoCertificates();
    };
    SystemSectionService.prototype.listCountryCodes = function () {
        return this.cryptoCertificateRepository.listCountryCodes();
    };
    SystemSectionService.prototype.listTunables = function () {
        return this.tunableRepository.listTunables();
    };
    SystemSectionService.prototype.listBootEnvironments = function () {
        var _this = this;
        return this.bootPoolRepository.loadBootEnvironments().then(function () { return _this.bootPoolRepository.listBootEnvironments(); });
    };
    SystemSectionService.prototype.saveBootEnvironment = function (bootEnvironment) {
        return this.bootPoolRepository.saveBootEnvironment(bootEnvironment);
    };
    SystemSectionService.prototype.keepBootEnvironment = function (bootEnvironment) {
        return this.setKeepBootEnvironment(bootEnvironment, true);
    };
    SystemSectionService.prototype.dontKeepBootEnvironment = function (bootEnvironment) {
        return this.setKeepBootEnvironment(bootEnvironment, false);
    };
    SystemSectionService.prototype.cloneBootEnvironment = function (bootEnvironment) {
        return this.bootPoolRepository.cloneBootEnvironment(bootEnvironment);
    };
    SystemSectionService.prototype.deleteBootEnvironment = function (bootEnvironment) {
        return this.bootPoolRepository.deleteBootEnvironment(bootEnvironment);
    };
    SystemSectionService.prototype.activateBootEnvironment = function (bootEnvironment) {
        return this.bootPoolRepository.activateBootEnvironment(bootEnvironment);
    };
    SystemSectionService.prototype.getBootVolumeConfig = function () {
        return this.bootPoolRepository.getBootPoolConfig();
    };
    SystemSectionService.prototype.scrubBootPool = function () {
        return this.bootPoolRepository.scrubBootPool();
    };
    SystemSectionService.prototype.saveAlertEmitter = function (alertEmitter) {
        return this.alertEmitterRepository.save(alertEmitter);
    };
    SystemSectionService.prototype.saveCertificate = function (certificate) {
        return this.cryptoCertificateRepository.saveCryptoCertificate(certificate);
    };
    SystemSectionService.prototype.listVolumes = function () {
        return this.volumeRepository.listVolumes();
    };
    SystemSectionService.prototype.listDisks = function () {
        var _this = this;
        if (!this.initialDiskAllocationPromise || this.initialDiskAllocationPromise.isRejected()) {
            this.initialDiskAllocationPromise = this.diskRepository.listDisks()
                .tap(function (disks) { return _this.volumeRepository.initializeDisksAllocations(_.map(disks, 'id')); });
        }
        return this.initialDiskAllocationPromise;
    };
    SystemSectionService.prototype.listAvailableDisks = function () {
        return this.diskRepository.listAvailableDisks();
    };
    SystemSectionService.prototype.listBootDisks = function () {
        return this.diskRepository.listBootDisks();
    };
    SystemSectionService.prototype.listVolumeSnapshots = function () {
        return this.volumeRepository.listSnapshots();
    };
    SystemSectionService.prototype.listVolumeDatasets = function () {
        return this.volumeRepository.listDatasets();
    };
    SystemSectionService.prototype.listShares = function () {
        return this.shareRepository.listShares();
    };
    SystemSectionService.prototype.listPeers = function () {
        return this.peerRepository.listPeers();
    };
    SystemSectionService.prototype.listReplications = function () {
        return this.replicationRepository.listReplications();
    };
    SystemSectionService.prototype.listDevicesWithClass = function (deviceClass) {
        return this.systemRepository.getDevices(deviceClass);
    };
    SystemSectionService.prototype.saveAlertFilters = function (alertFilters) {
        var _this = this;
        return _.map(alertFilters, function (alertFilter) { return _this.alertFilterRepository.save(alertFilter); });
    };
    SystemSectionService.prototype.deleteAlertFilters = function (alertFilters) {
        var _this = this;
        return _.map(alertFilters, function (alertFilter) { return _this.alertFilterRepository.delete(alertFilter); });
    };
    SystemSectionService.prototype.loadExtraEntries = function () {
        return undefined;
    };
    SystemSectionService.prototype.loadSettings = function () {
        return undefined;
    };
    SystemSectionService.prototype.loadOverview = function () {
        return undefined;
    };
    SystemSectionService.prototype.setKeepBootEnvironment = function (bootEnvironment, keep) {
        bootEnvironment.keep = keep;
        return this.bootPoolRepository.saveBootEnvironment(bootEnvironment);
    };
    SystemSectionService.prototype.getNextSequenceForStream = function (streamId) { };
    return SystemSectionService;
}(abstract_section_service_ng_1.AbstractSectionService));
exports.SystemSectionService = SystemSectionService;
